/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

--*/

#include "PhidgetDebug.h"
#include <math.h>

//////////////////////////////////////////////////////////////////////////
// Macros

#define GetPointerStringSize() (sizeof(void*) + 2)
#define GetIntegerStringSize(i) ((int) log10( pow(2, sizeof(i)*8) ) + 1)

//////////////////////////////////////////////////////////////////////////
// Forward declarations

char* phidgetPlugin_GetPhidgetDescription_Special(Phidget* pPhidget);

//////////////////////////////////////////////////////////////////////////
// Debugging functions

// TODO: Kommentieren
char* phidgetPlugin_GetPhidgetDescription(Phidget* pPhidget)
{
	char *pcDescription = NULL, *pcDescriptionSpecial = NULL;
	int iSize = 0;

	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetPlugin_GetPhidgetDescription for pPhidget."));
		return NULL;
	}

	// get the non-general info about the phidget
	pcDescriptionSpecial = phidgetPlugin_GetPhidgetDescription_Special(pPhidget);
	if(pcDescriptionSpecial == NULL) pcDescriptionSpecial = "";

	// allocate memory (for worst-case size of the string - Check this number every time a new Phidget is supported)
	iSize = // the general information. Text is 20 characters long (per line) + string represenations for integers/pointers // + special info size
		8*20 + 3*GetPointerStringSize() + 3*GetIntegerStringSize(pPhidget->iSemaphoreAttach) + 
		GetIntegerStringSize(pPhidget->deviceClass) + GetIntegerStringSize(pPhidget->id) + 
		strlen(pcDescriptionSpecial) + 1;
	pcDescription = malloc(iSize);
	if(pcDescription == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_GetPhidgetDescription.", iSize));
		return NULL;
	}

	// put string together
	sprintf(pcDescription,
		"pPhidget: %p\n\
\thandle: %p\n\
\tid: %d\n\
\tdeviceClass: %d\n\
\tpNextLink: %p\n\
\tAttach: %d\n\
\tDetach: %d\n\
\tError: %d\n\
%s",
		pPhidget, pPhidget->handle, pPhidget->id, pPhidget->deviceClass, pPhidget->pNextLink,
		pPhidget->iSemaphoreAttach, pPhidget->iSemaphoreDetach, pPhidget->iSemaphoreError,
		pcDescriptionSpecial);

	// clean up
	if(pcDescriptionSpecial != NULL && pcDescriptionSpecial != "")
		free(pcDescriptionSpecial);

	return pcDescription;
}

// TODO: Kommentieren
char* phidgetPlugin_GetPhidgetDescription_Special(Phidget* pPhidget)
{
	char* pcDescription = NULL;
	int iSize = 0;

	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetPlugin_GetPhidgetDescription_Special for pPhidget."));
		return NULL;
	}

	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_INTERFACEKIT:
		iSize = 3 * (20 + GetIntegerStringSize(pPhidget->iSemaphoreAttach));
		pcDescription = malloc(iSize);
		if(pcDescription == NULL)
		{
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_GetPhidgetDescription_Special.", iSize));
			return NULL;
		}
		sprintf(pcDescription, 
"\tinput change: %d\n\
\toutput change: %d\n\
\tsensor change: %d",
			((PhidgetIfKit*)pPhidget)->iSemaphoreInputChange,
			((PhidgetIfKit*)pPhidget)->iSemaphoreOutputChange,
			((PhidgetIfKit*)pPhidget)->iSemaphoreSensorChange);
		break;

	case PHIDCLASS_RFID:
		iSize = 3 * (20 + GetIntegerStringSize(pPhidget->iSemaphoreAttach));
		pcDescription = malloc(iSize);
		if(pcDescription == NULL)
		{
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_GetPhidgetDescription_Special.", iSize));
			return NULL;
		}
		sprintf(pcDescription, 
"\toutput change: %d\n\
\ttag found: %d\n\
\ttag lost: %d",
			((PhidgetRFID*)pPhidget)->iSemaphoreOutputChange,
			((PhidgetRFID*)pPhidget)->iSemaphoreTagFound,
			((PhidgetRFID*)pPhidget)->iSemaphoreTagLost);
		break;

	case PHIDCLASS_SERVO:
		iSize = 20 + GetIntegerStringSize(pPhidget->iSemaphoreAttach);
		pcDescription = malloc(iSize);
		if(pcDescription == NULL)
		{
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_GetPhidgetDescription_Special.", iSize));
			return NULL;
		}
		sprintf(pcDescription,"\tposition change: %d",
			((PhidgetServo*)pPhidget)->iSemaphorePositionChange);
		break;

	case PHIDCLASS_ACCELEROMETER:
		iSize = 20 + GetIntegerStringSize(pPhidget->iSemaphoreAttach);
		pcDescription = malloc(iSize);
		if(pcDescription == NULL)
		{
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_GetPhidgetDescription_Special.", iSize));
			return NULL;
		}
		sprintf(pcDescription,"\taxis change: %d",
			((PhidgetAccel*)pPhidget)->iSemaphoreAccelerationChange);
		break;

	case PHIDCLASS_MOTORCONTROL:
		iSize = 3 * (20 + GetIntegerStringSize(pPhidget->iSemaphoreAttach));
		pcDescription = malloc(iSize);
		if(pcDescription == NULL)
		{
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_GetPhidgetDescription_Special.", iSize));
			return NULL;
		}
		sprintf(pcDescription, 
"\tvelocity change: %d\n\
\tcurrent change: %d\n\
\tinput change: %d",
			((PhidgetMotCon*)pPhidget)->iSemaphoreVelocityChange,
			((PhidgetMotCon*)pPhidget)->iSemaphoreCurrentChange,
			((PhidgetMotCon*)pPhidget)->iSemaphoreInputChange);
		break;

	case PHIDCLASS_ADVANCEDSERVO:
		iSize = 3 * (20 + GetIntegerStringSize(pPhidget->iSemaphoreAttach));
		pcDescription = malloc(iSize);
		if(pcDescription == NULL)
		{
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetPlugin_GetPhidgetDescription_Special.", iSize));
			return NULL;
		}
		sprintf(pcDescription, 
"\tvelocity change: %d\n\
\tcurrent change: %d\n\
\tposition change: %d",
			((PhidgetAdvServo*)pPhidget)->iSemaphoreVelocityChange,
			((PhidgetAdvServo*)pPhidget)->iSemaphoreCurrentChange,
			((PhidgetAdvServo*)pPhidget)->iSemaphorePositionChange);
		break;

	case PHIDCLASS_TEXTLCD:
		pcDescription = "";
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "Unknown Phidget type %i in phidgetPlugin_PrintPhidget.", phidget_GetDeviceClass(pPhidget)));
		pcDescription = "";
		break;
	}

	return pcDescription;
}
