/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

--*/

#include "xmutex.h"

//////////////////////////////////////////////////////////////////////////
// function definitions

// creation of a platform independent mutex
BOOL xmutex_create(xmutex_t* pMutex)
{
#ifdef _WINDOWS
	InitializeCriticalSection(pMutex);
	return TRUE;
#else
	int iResult;
	pthread_mutexattr_t attr;

	pthread_mutexattr_init(&attr);
	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
	iResult = pthread_mutex_init(pMutex, &attr);
	pthread_mutexattr_destroy(&attr);
	return iResult == 0;
#endif
}

BOOL xmutex_destroy(xmutex_t* pMutex)
{
#ifdef _WINDOWS
	DeleteCriticalSection(pMutex);
	return TRUE;
#else
	return pthread_mutex_destroy(pMutex) == 0;
#endif
}

BOOL xmutex_lock(xmutex_t* pMutex)
{
#ifdef _WINDOWS
	EnterCriticalSection(pMutex);
	return TRUE;
#else
	return pthread_mutex_lock(pMutex) == 0;
#endif
}

BOOL xmutex_unlock(xmutex_t* pMutex)
{
#ifdef _WINDOWS
	LeaveCriticalSection(pMutex);
	return TRUE;
#else
	return pthread_mutex_unlock(pMutex) == 0;
#endif
}
